from machine import DAC, Pin
import math
import time

# Inicjalizacja DAC na pinie GPIO25
dac = DAC(Pin(25, Pin.OUT))

# Inicjalizacja przycisku na GPIO14
przycisk = Pin(14, Pin.IN, Pin.PULL_UP)

# Inicjalizacja zmiennych
tryb = 1
ostatni_czas = 0  # Zmienna do debouncingu
 
# Funkcja obsługi przerwania
def zmien_tryb(pin):
    global tryb, ostatni_czas
    aktualny_czas = time.ticks_ms()  # Pobranie czasu w milisekundach
 
    # Debouncing: sprawdzenie, czy od ostatniego wywołania minęło co najmniej 200 ms
    if time.ticks_diff(aktualny_czas, ostatni_czas) > 200:
        ostatni_czas = aktualny_czas  # Aktualizacja ostatniego czasu
        tryb += 1
        if tryb > 8:
            tryb = 1
        print(tryb)
 
# Konfiguracja przycisku
przycisk.irq(trigger=Pin.IRQ_FALLING, handler=zmien_tryb)
# Definicje funkcji dla różnych sygnałów
def sygnal_sinusoidalny1():
    SAMPLE_RATE = 1000
    NUM_SAMPLES = 100
    AMPLITUDE = 127
    OFFSET = 128
    sine_wave = [
        int(OFFSET + AMPLITUDE * math.sin(2 * math.pi * i / NUM_SAMPLES))
        for i in range(NUM_SAMPLES)
    ]
    while tryb == 1:
        for value in sine_wave:
            dac.write(value)
            time.sleep_us(1_000_000 // SAMPLE_RATE)

def sygnal_sinusoidalny2():
    SAMPLE_RATE = 100
    NUM_SAMPLES = 80
    AMPLITUDE = 50
    OFFSET = 128
    sine_wave = [
        int(OFFSET + AMPLITUDE * math.sin(2 * math.pi * i / NUM_SAMPLES))
        for i in range(NUM_SAMPLES)
    ]
    while tryb == 2:
        for value in sine_wave:
            dac.write(value)
            time.sleep_us(1_000_000 // SAMPLE_RATE)

def sygnal_trojkatny():
    amplituda = 255
    krok = 5
    opoznienie = 0.01
    wartosc = 0
    kierunek = 1
    while tryb == 3:
        dac.write(wartosc)
        wartosc += kierunek * krok
        if wartosc >= amplituda:
            wartosc = amplituda
            kierunek = -1
        elif wartosc <= 0:
            wartosc = 0
            kierunek = 1
        time.sleep(opoznienie)

def sygnal_kwadratowy1():
    amplituda = 255
    poziom_niski = 0
    okres = 0.1
    while tryb == 4:
        dac.write(amplituda)
        time.sleep(0.01)
        dac.write(poziom_niski)
        time.sleep(2)

def sygnal_kwadratowy2():
    amplituda = 255
    poziom_niski = 0
    okres = 0.5
    while tryb == 5:
        dac.write(amplituda)
        time.sleep(okres / 2)
        dac.write(poziom_niski)
        time.sleep(okres / 2)

def sygnal_zmienny():
    parametry = [
        {"amplituda": 128, "czestotliwosc": 1},
        {"amplituda": 255, "czestotliwosc": 2},
        {"amplituda": 64, "czestotliwosc": 0.5},
    ]
    czas_zmiany = 2
    probkowanie = 100
    indeks = 0
    while tryb == 6:
        amplituda = parametry[indeks]["amplituda"]
        czestotliwosc = parametry[indeks]["czestotliwosc"]
        for i in range(int(czas_zmiany * probkowanie)):
            kat = 2 * math.pi * czestotliwosc * (i / probkowanie)
            wartosc = int((math.sin(kat) + 1) * (amplituda / 2))
            dac.write(wartosc)
            time.sleep(1 / probkowanie)
        indeks = (indeks + 1) % len(parametry)

def generuj_sinus_z_wzrostem_amplitudy():
    """
    Generuje sinusoidę, w której co 3 sekundy amplituda chwilowo wzrasta.
    """
    SAMPLE_RATE = 1000   # Częstotliwość próbkowania
    NUM_SAMPLES = 100    # Liczba próbek na cykl
    BASE_AMPLITUDE = 50 # Podstawowa amplituda
    OFFSET = 128         # Offset dla sygnału DAC (środek)
    BOOST_AMPLITUDE = 127 # Zwiększona amplituda podczas skoku
    BOOST_DURATION = 1    # Czas trwania zwiększonej amplitudy (w sekundach)
    BOOST_INTERVAL = 3    # Czas między wzrostami amplitudy (w sekundach)
    
    # Generowanie jednej pełnej fali sinusoidalnej
    sine_wave_base = [
        int(OFFSET + BASE_AMPLITUDE * math.sin(2 * math.pi * i / NUM_SAMPLES))
        for i in range(NUM_SAMPLES)
    ]
    
    sine_wave_boost = [
        int(OFFSET + BOOST_AMPLITUDE * math.sin(2 * math.pi * i / NUM_SAMPLES))
        for i in range(NUM_SAMPLES)
    ]

    # Główna pętla generowania sygnału
    last_boost_time = time.ticks_ms()  # Ostatni czas skoku amplitudy
    while tryb==7:
        current_time = time.ticks_ms()
        # Sprawdzanie, czy minął czas na chwilowy wzrost amplitudy
        if time.ticks_diff(current_time, last_boost_time) > BOOST_INTERVAL * 1000:
            # Generowanie sinusoidy ze zwiększoną amplitudą
            for value in sine_wave_boost:
                dac.write(value)
                time.sleep_us(1_000_000 // SAMPLE_RATE)
            #time.sleep(BOOST_DURATION)  # Pauza po skoku amplitudy
            last_boost_time = time.ticks_ms()  # Aktualizacja czasu ostatniego wzrostu
        else:
            # Generowanie sinusoidy z podstawową amplitudą
            for value in sine_wave_base:
                dac.write(value)
                time.sleep_us(1_000_000 // SAMPLE_RATE)

def modulacja():
        # Parametry modulacji
    #dac = DAC(Pin(25))              # DAC1 (GPIO25)
    sample_count = 1000             # Liczba próbek
    amplitude = 127                 # Amplituda (środek w zakresie 0-255)
    carrier_freq = 1000            # Częstotliwość fali nośnej (w Hz)
    modulation_freq = 10           # Częstotliwość modulacji (w Hz)
    modulation_index = 5           # Indeks modulacji

    sample_rate = 44100            # Częstotliwość próbkowania (Hz)
    delay = 1 / sample_rate

    while tryb==8:
        for n in range(sample_count):
            t = n / sample_rate

            # Częstotliwość chwilowa
            instant_freq = carrier_freq + modulation_index * math.sin(2 * math.pi * modulation_freq * t)
            # Faza chwilowa
            phase = 2 * math.pi * instant_freq * t
            # Obliczenie wartości próbki
            sample = int(amplitude * math.sin(phase) + 128)

            # Wysłanie próbki do DAC
            dac.write(sample)

            # Kontrola szybkości próbkowania
            time.sleep(delay)



# Główna pętla sterująca
while True:
    if tryb == 1:
        sygnal_sinusoidalny1()
    elif tryb == 2:
        sygnal_sinusoidalny2()
    elif tryb == 3:
        sygnal_trojkatny()
    elif tryb == 4:
        sygnal_kwadratowy1()
    elif tryb == 5:
        sygnal_kwadratowy2()
    elif tryb == 6:
        sygnal_zmienny()
    elif tryb==7:
        generuj_sinus_z_wzrostem_amplitudy()
    elif tryb==8:
        modulacja()
        


